rm(list = ls()) # To clear all
# set path
setwd("D://het_instrument/Code/estimation/main")

# ------------------ Load packages and source functions ------------------
# load packages and source functions
source("./general/organization_code/load_packages.R")
source("./general/source.R")
sourceFolder("./general", FALSE, FALSE)
sourceFolder("../simulation", FALSE, FALSE)
sourceFolder("../ms_switch/", FALSE, FALSE)
sourceFolder("../proxy/", FALSE, FALSE)

# set up parallel --> some parallel implementation is highly recommended. This code runs an MPI-cluster on a suitable machine
nb_cores <- 10 # number of cores to be used
cluster <- makeCluster(nb_cores, type = "SOCK")
registerDoSNOW(cluster)

# ----------------------------- Specification of Model -----------------------------------
path_res <- paste0(getwd(),"/")
if (!dir.exists(path_res)){dir.create(path_res, recursive=TRUE) }
save_res <- paste0(path_res,"sim_baseline.RData")
cat(save_res)

aug_var <- TRUE # estimate "augmented" model with instrument as endogenous variable
seed <- 4131643 # Set the seed for reproducible results
sims <-   500 # number of simulation repetitions
Samp <- 200 # 200, 500 set sample size
ins_dim <- 1 # dimension of instrument: 1 or 2 
# Specifiy instrument strength
weak_ins <- FALSE # additional degrees of strength
if (weak_ins == TRUE){  # additional beta for weak instrument case
  strength_var <- c(0, 0.16, 0.23, 0.35, 0.53, 0.72, 1)
} else {
  strength_var <- c(0, 0.35, 0.72, 1)
}
endo_var <- c(0, 0.05, 0.17, 0.27,  0.37)
# generate beta (strength and endogeneity of instrument)
com_beta <- as.matrix(expand.grid(strength_var,endo_var))
com_beta <- com_beta[com_beta[,1]>=com_beta[,2],][-c(11),]
selec_beta <- 4 # change this for different betas --> on parallel cluster this was specified as array job
beta_true <- matrix(rbind(c(com_beta[selec_beta,1],com_beta[selec_beta,2],0), c(0.72,0.0,0))[1:ins_dim,], nrow=ins_dim)

# variance noise term of instrument
lambda_nu <- list(as.matrix(matrix(c(1,0.2,0.2,1),2)[1:ins_dim, 1:ins_dim]), as.matrix(matrix(c(12,3,3,12),2)[1:ins_dim, 1:ins_dim]), as.matrix(matrix(c(20,5,5,20),2)[1:ins_dim, 1:ins_dim]))

K <-  3 # dimension of VAR
lag_length <- 1 # Lag length
states <- 2 # States of Markov Chain
horizon_imp <- 48 # Horizon to be used in impulse responses

# Different specifications of simulation design
states_mis <- FALSE # misspecified number of states --> generate data with 3 states, estimate 2 states
set_zero <- FALSE # set fraction of values in instrument to zero
fail_constB <- FALSE # add random error to B matrix
gen_ST <- FALSE # generate data by ST-in-variances
gen_ST_IP <- FALSE # generate data by ST-in-variances
gen_exog <- FALSE # generate data by exogenous change-in-variances
conf_shift <- FALSE # confounding shift in variance
lambda_alt <- FALSE # alternative values of Lambda

code <-  c(0,0,0) # transformation of variables (0 = no transformation, 1 = logs, 2 = first diff)
# Put all information in spec-list element which is passed to the algorithm below
LM <- 1 # starting value for Lambda
spec <- list(states=states, ins_dim = ins_dim, fail_constB= fail_constB, gen_ST=gen_ST, gen_ST_IP=gen_ST_IP, iterations = length(LM), set_zero=set_zero, lag=lag_length, horizon=horizon_imp, rep=500, boot="no", instrument =TRUE, pos_instr = 1, unres=FALSE, sim=TRUE)
spec$const = matrix(1,spec$states,1)

# specify setup for different simulation designs
if (set_zero == TRUE){
  spec$ins_drive = FALSE
  spec$share0 <- 30 # share of zeros in instrument 
} else {
  spec$ins_drive = TRUE # does instrument drive states?
}
if (gen_exog == TRUE){
  spec$bp <- 0.5 # set breakpoint
}

source("../simulation/organization_code/results_array.R") # empty containers for results
coef_true <-  make_coef(K, lag_length,1) # get model coefficients
mean <-  make_coef(K, lag_length,2) # process mean
B_true <- make_coef(K, lag_length,3)$B_true # impact effects matrix
Lambda_true <- make_coef(K, lag_length,3, conf_shift = conf_shift)$Lambda_true # Variances of struc. shocks

if (states_mis == FALSE){
  P_true <- matrix(c(0.975,0.05,0.025,0.95),2,2) # matrix of transition probabilities
} else{
  P_true <- matrix(c(0.97,0.025, 0.25,0.015,0.95, 0.25, 0.01, 0.025,0.5),3,3) # matrix of transition probabilities
}
#  Calculate the true VAR-coefficients
for (k in 1:lag_length){
  if (k == 1){
    Theta_true <- cbind(mean, coef_true[[k+1]])
  } else if (k > 1){
    Theta_true <- cbind(Theta_true, coef_true[[k+1]])
  }
}

# ------------------- GENERATE THE DATA FOR THE SIMULATION -------------------------------------- #
set.seed(seed) # set seed

for (i in 1:sims){ 
  if (gen_ST == TRUE || gen_ST_IP == TRUE){
    data_load <- data_smooth(Samp, K, coef_true, mean, P_true, B_true, Lambda_true, lambda_nu, beta_true,  spec)
  } else if (gen_exog == TRUE){
    data_load <- data_exogenous2(Samp,K, coef_true, mean, B_true, Lambda_true, beta_true, lambda_nu, spec)
  } else {
    data_load <- data_mswitch(Samp, coef_true, mean, P_true,  B_true, Lambda_true,lambda_nu, beta_true=beta_true,spec)
  }
  
  # write data into y and u vectors to work with, also save into arrays so that data is not overwritten in rounds of simulation
  y <- as.matrix(data_load$y)
  y_aug <- cbind(y, data_load$instrument)
  
  struc_shocks_sims[,,i] <- data_load$struc_shocks
  u_sims[,,i] <- data_load$resid
  instrument_array[,,i] <- as.matrix(data_load$instrument)
  # save transition function in smooth transition DGP case
  states_dgp[,,i] <- as.matrix(data_load$states_dgp)
  
  # input arguments of function ylags(y,lag_length) are data matrix and number of desired lags, output is matrix with lags
  y_lag_array[,,i]<- y_lag_cr(y, lag_length)$lags
  y_lag_array_aug[,,i]<- y_lag_cr(y_aug, lag_length)$lags
  
  y_start_array[,,i] <- (y[(1:lag_length),])
  y <- as.matrix(y[-(1:lag_length),])
  y_sims[,,i] <- y
  y_start_array_aug[,,i] <- (y_aug[(1:lag_length),])
  y_aug <- as.matrix(y_aug[-(1:lag_length),])
  y_sims_aug[,,i] <- y_aug
  
  #generate Z matrix with constant
  con <- matrix(1,Samp-lag_length,1)
  Z_array[,,i] <- cbind(con,y_lag_array[,,i])
  Z_array_aug[,,i] <- cbind(con,y_lag_array_aug[,,i])
  rm(con)
} # end of loop through Simulations
rm(data_load)
# generate the data array depend on DGP
data_complete <- list(Samp=Samp, y_sims = y_sims, ins_sims= instrument_array, struc_shocks_sims=struc_shocks_sims, u_sims=u_sims, y_start = y_start_array, states_dgp = states_dgp)
# random date (other code needs this, but irrelevant for simulation)
rnd_date <- substr(seq(as.Date("1970/1/1"), by = "month", length.out = Samp),1,7)
date_list <-  list(date_var= rnd_date, date_ins=rnd_date)

# -------------------------
# 1. Markov Switching Model
# -------------------------

correlations <- t(sapply(1:sims, function(i) cor(data_complete$ins_sims[,,i], data_complete$struc_shocks_sims[,,i])))
matrix(colMeans(correlations), nrow=ins_dim) # mean of correlations of instrument and first struc shock

log_path <- "log.txt" # where to log
file.create(log_path)

########################################### STATE-INVARIANT B ##################################################
invarB <- vector(mode = "list", length = sims)

invarB <- foreach(i=1:sims, .packages=c("Rsolnp","matrixcalc", "pracma","miscTools", "MASS")) %dopar% { # loop over the simulations
  cat(paste("invarB: Starting iteration",i,"\r\n"),  file=log_path, append=TRUE)
  spec$restric <- "invarB" # no restrictions of B
  spec$unres <-  FALSE   # estimation of a reduced form model --> no "structure" on covariance matrix 
  spec$clean <- TRUE # restrictions on autoregessive part of models in rows or columns for instrument
  invarB[[i]] <- ms_var(spec, y=as.matrix(y_sims_aug[,,i]), Z=as.matrix(Z_array_aug[,,i]), LM, Samp, K=dim(as.matrix(y_sims_aug[,,i]))[2], y_start = as.matrix(y_start_array_aug[,,i]), log_path, code=code, date_list, verbose=FALSE, start_val= c(), aug_var=TRUE)	
}
invarB_res <- list() 
invarB_res <- foreach(i=1:sims, .packages=c("Rsolnp","matrixcalc", "pracma","miscTools", "MASS")) %dopar% { # loop over the simulations
  cat(paste("invarB_res: Starting iteration",i,"\r\n"),  file=log_path, append=TRUE)
  # name of model specification
  spec$restric <- "invarB-res" # last column of B restricted
  # estimation of a reduced form model --> no "structure" on covariance matrix 
  spec$unres <-  FALSE   # estimation of a reduced form model --> no "structure" on covariance matrix 
  spec$clean <- TRUE # restrictions on autoregessive part of models in rows or columns for instrument
  invarB_res[[i]] <- ms_var(spec, y=as.matrix(y_sims_aug[,,i]), Z=as.matrix(Z_array_aug[,,i]), LM, Samp,  K=dim(as.matrix(y_sims_aug[,,i]))[2], as.matrix(y_start_array_aug[,,i]), log_path, code=code, date_list, verbose=FALSE, start_val=c(),aug_var )
}
############################################ PH-VAR #############################################################
ph_var <- list() 
ph_var <- foreach(i=1:sims, .packages=c("Rsolnp","matrixcalc", "pracma","miscTools", "MASS")) %dopar% { # loop over the simulations
  cat(paste("PH-VAR: Starting iteration",i,"\r\n"),  file=log_path, append=TRUE)
  # name of model specification
  spec$restric <- "ph-var"
  spec$lenb <-  1 # how many elemts of beta are estimated
  spec$unres <-  FALSE   # estimation of a reduced form model --> no "structure" on covariance matrix 
  spec$clean <- TRUE # restrictions on autoregessive part of models in rows or columns for instrument
  ph_var[[i]] <- ms_var(spec, y=as.matrix(y_sims_aug[,,i]), Z=as.matrix(Z_array_aug[,,i]), LM, Samp,  K=dim(as.matrix(y_sims_aug[,,i]))[2], as.matrix(y_start_array_aug[,,i]), log_path, code=code, date_list, verbose=FALSE, start_val=c(),aug_var, b_method="rnd" )
}
########################################### H-VAR ##############################################################
h_var <- list() 
h_var <- foreach(i=1:sims, .packages=c("Rsolnp","matrixcalc", "pracma","miscTools", "MASS")) %dopar% { # loop over the simulations
  cat(paste("HVAR: Starting iteration",i,"\r\n"),  file=log_path, append=TRUE)
  # name of model specification
  spec$restric <- "h-var-res"
  spec$unres <-  FALSE   # estimation of a reduced form model --> no "structure" on covariance matrix 
  spec$clean <- TRUE # restrictions on autoregessive part of models in rows or columns for instrument
  spec$trend <- FALSE # trend in VAR
  h_var[[i]] <- ms_var(spec, y=as.matrix(y_sims_aug[,,i]), Z=as.matrix(Z_array_aug[,,i]), LM, Samp, K=dim(as.matrix(y_sims_aug[,,i]))[2], as.matrix(y_start_array_aug[,,i]), log_path, code=code, date_list, verbose=FALSE, start_val=c(),aug_var, b_method="rnd")
}
########################################### PROXY-VAR ##############################################################
pr_var <- list()
# specify a spec-list for Proxy VAR
spec_proxy <-  list(lag=lag_length, horizon=horizon_imp, rep=500, boot="no", sim=TRUE)
spec_proxy$restric <- "proxy"
pr_var <- foreach(i=1:sims, .export=ls(.GlobalEnv), .packages=c("Rsolnp","matrixcalc", "pracma","miscTools", "MASS")) %dopar% { # loop over the simulations
  cat(paste("PrVAR: Starting iteration",i,"\r\n"),  file=log_path, append=TRUE)
  spec_proxy$instrument <-  instrument_array[,,i]
  spec_proxy$pos_instr <- 1
  pr_var[[i]] <- proxy_var(spec = spec_proxy, y=as.matrix(y_sims[,,i]), Z=as.matrix(Z_array[,,i]), y_start =as.matrix(y_start_array[,,i]), Samp, K=dim(as.matrix(y_sims[,,i]))[2], date_list)
}
########################################### Cholesky-VAR A LA Plagborg/Moeller ##############################################################
plag_var <- list()
plag_var <- foreach(i=1:sims, .packages=c("Rsolnp","matrixcalc", "pracma","miscTools", "MASS")) %dopar% { # loop over the simulations
  cat(paste("Plag-VAR: Starting iteration",i,"\r\n"),  file=log_path, append=TRUE)
  plag_var[[i]] <- var_plag_moel(spec = spec,y=as.matrix(y_sims_aug[,,i]), Z=as.matrix(Z_array_aug[,,i]),  Samp, K=dim(as.matrix(y_sims_aug[,,i]))[2], imp_scale = pr_var[[i]]$B)
}
# summarize results: calculate p-values, MSEs, etc.
source("../simulation/organization_code/sim_meta.R")

